@@ -5,12 +5,12 @@ use Tk::Xlib;
 use Tk::After;
 use Tk::Animation;
 use Tk::Font;
-use X11::Protocol;
 
 use Tk::SlideShow::Dict;
 use Tk::SlideShow::Placeable;
 use Tk::SlideShow::Diapo;
 use Tk::SlideShow::Sprite;
+use Tk::SlideShow::Oval;
 use Tk::SlideShow::Link;
 use Tk::SlideShow::Arrow;
 use Tk::SlideShow::DblArrow;
@@ -31,10 +31,11 @@ sub pile {
 
 #------------------------------------------------
 package Tk::SlideShow;
-
-use vars qw($VERSION);
-
-$VERSION='0.05';
+require Exporter;
+use vars qw($VERSION @EXPORT @ISA);
+@ISA=qw(Exporter);
+@EXPORT=qw(template);
+$VERSION='0.07';
 
 my ($can,$H,$W,$xprot,$present);
 my $mainwindow;
@@ -42,11 +43,13 @@ my $mode = 'X11';
 my $family = "charter";
 use vars qw($inMainLoop $nextslide $jumpslide);
 $nextslide = 0;
+
 sub var_getset{
   my ($s,$k,$v) = @_;
   if (defined $v) {$s->{$k} = $v; return $s;}
   else            {               return $s->{$k} ;}
 };
+
 sub family {
   my ($class,$newfamily) = @_;
   if (defined $newfamily) {$family = $newfamily;}
@@ -72,6 +75,14 @@ sub canvas {return $can }
 sub h { return $H}
 sub w { return $W}
 
+sub present_start { var_getset((shift),'present_start',@_)};
+sub diapo_start   { var_getset((shift),'diapo_start',@_)};
+my $steps = 50;
+sub steps { my ($s,$v) = @_;
+	    return $steps unless defined $v;
+	    $steps = $v;
+	    return $s}
+
 
 sub title_ne {
   my ($s,$texte) = @_;
@@ -89,11 +100,14 @@ my $current_item = "";
 
 sub enter {
   $current_item = ($can->gettags('current'))[0]; 
-  # print "entering $current_item\n"; 
+#  my $s = Tk::SlideShow::Dict->Get($current_item);
+#  print "entering $current_item\n"; 
+#  $can->configure(-cursor, 'hand2');
 }
 sub leave {
-  # print "leaving $current_item\n"; 
+#  print "leaving $current_item\n"; 
   $current_item = "";
+#  $can->configure(-cursor, 'xterm');
 }
 
 sub current_item {
@@ -115,8 +129,13 @@ sub init {
   $mainwindow = $m;
   $present = bless { 'current' => 0, 'mw' => $m, 'fond'=>'ivory',
 		   'slides_names' => {}};
-  # util pour forcer le dplaceement de la souris (pointer)
-  $xprot = X11::Protocol->new();
+  # This following part is there to force pointer to move
+  # It is used for placing anchor of arrows.
+  eval q{
+      use X11::Protocol;
+      $xprot = X11::Protocol->new();
+  };
+  warn $@ if $@;
   $H = $h || $m->Display->ScreenOfDisplay->HeightOfScreen;
   $W = $w || $m->Display->ScreenOfDisplay->WidthOfScreen;
   print ("H=$H, W=$W\n");
@@ -202,11 +221,22 @@ sub postscript {
 		   -pagewidth => "21.0c",
 		   -rotate => 1);
 }
+sub photos {
+    my $title = $mainwindow->title;
+    print "title $title\n";
+    my $nu = (lc $title).".00";
+    $nu++ while -f "$nu.gif";
+    my $cmd = "xwd -name $title| xwdtopnm | ppmtogif > $nu.gif";
+    print "command : $cmd\n";
+    system $cmd;
+
+}
 
 #internals
 sub warppointer {
   my ($x,$y) = @_;
-  $xprot->WarpPointer(0, hex($can->id), 0, 0, 0, 0, $x, $y);
+  $xprot->WarpPointer(0, hex($can->id), 0, 0, 0, 0, $x, $y)
+      if $xprot;
 }
 
 # this sub create a popup window with key binding help
@@ -266,24 +296,27 @@ sub init_bindings {
   $m->Tk::bind('Tk::SlideShow','<q>', sub {$m->destroy; exit});
   addkeyhelp('Press q','to quit');
   $m->Tk::bind('Tk::SlideShow','<p>', \&postscript);
+  $m->Tk::bind('Tk::SlideShow','<P>', \&photos);
+  $m->Tk::bind('Tk::SlideShow','<Up>', sub { print "curitem=$current_item"});
   $m->Tk::bind('Tk::SlideShow','<h>', \&posthelp);
   addkeyhelp('Press h','to get this help');
 }
 
 
 #internals
-sub trace_fond {
-  shift;
-  my $m = $mainwindow;
-  if (ref($present->bg) eq 'CODE') {
-    &{$present->bg};
-  } else {
-    $can->configure(-background, $present->bg);
+{ my $repeat_id;
+  sub trace_fond {
+    shift;
+    my $m = $mainwindow;
+    if (ref($present->bg) eq 'CODE') {
+      &{$present->bg};
+    } else {
+      $can->configure(-background, $present->bg);
+    }
+    $repeat_id->cancel if defined $repeat_id;
+    default_footer();
+    $repeat_id = $m->repeat(5000,\&default_footer);
   }
-  my $t = $present->currentName. "(".($present->current+1)."/". $present->nbslides.")";
-
-  $can->createText(10,$H - 10,'-text',$t,-anchor,'sw',
-		-font, Tk::SlideShow->f1);
 }
 #internals
 sub wait {
@@ -300,6 +333,8 @@ sub wait {
 sub clean { 
   my $class = shift;
   $can->delete('all'); 
+#  print "Afters : ".join('     ',$can->after('info'))."\n";
+  for ($can->after('info')) { $can->Tk::after('cancel',$_);}
   $present->{'action'}= [];
   $present->{'save_action'}= [];
   Tk::SlideShow::Placeable->Clean;
@@ -329,9 +364,16 @@ sub arrive {
   return  unless $mode eq 'X11';
   for my $tag (@tags) {
     if (ref($tag) eq 'ARRAY') {
-      for (@$tag) {$can->move($_,-$dx,-$dy) if visible($can,$_);}
+      for (@$tag) {
+	$can->move($_,-$dx,-$dy) if visible($can,$_);
+	my $spri =  Tk::SlideShow::Dict->Get($_);
+	for my $l ($spri->links) {$l->hide;}
+      }
     } else { 
       $can->move($tag,-$dx,-$dy) if visible($can,$tag);
+      my $spri =  Tk::SlideShow::Dict->Get($tag);
+      for my $l ($spri->links) {$l->hide;}
+
     }
     push @{$present->{'action'}},[$tag,$maniere,$dx,$dy];
   }
@@ -353,23 +395,40 @@ sub shiftaction {
   @_ = (@$a);
   my $tag = shift;
   my $maniere = shift;
-  my $step = 50;
+  my $step = Tk::SlideShow->steps;
   $maniere eq 'smooth'  and 
     do {
       my ($dx,$dy) = @_;
       for(my $i=0;$i<$step;$i++){
 	if (ref($tag) eq 'ARRAY') {
-	  for (@$tag) {$c->move($_,$dx/$step,$dy/$step);}
-	} else { $c->move($tag,$dx/$step,$dy/$step);}
+	  for (@$tag) {
+	    $c->move($_,$dx/$step,$dy/$step);
+	    my $spri = Tk::SlideShow::Dict->Get($_);
+	    for my $l ($spri->links) {$l->show;}
+	  }
+	} else { 
+	  $c->move($tag,$dx/$step,$dy/$step);
+	  my $spri = Tk::SlideShow::Dict->Get($tag);
+	  for my $l ($spri->links) {$l->show;}
+	}
 	$c->update;
       }
+
     };
   $maniere eq 'direct' and 
     do {
       my ($dx,$dy) = @_;
       if (ref($tag) eq 'ARRAY') {
-	for (@$tag) {$c->move($_,$dx,$dy);}
-      } else { $c->move($tag,$dx,$dy);}
+	for (@$tag) {
+	  $c->move($_,$dx,$dy);
+	  my $spri = Tk::SlideShow::Dict->Get($_);
+	  for my $l ($spri->links) {$l->show;}
+	}
+      } else { 
+	$c->move($tag,$dx,$dy);
+	my $spri = Tk::SlideShow::Dict->Get($tag);
+	for my $l ($spri->links) {$l->show;}
+      }
       $c->update;
     };
   $maniere eq 'a_chpos' and 
@@ -396,14 +455,22 @@ sub unshiftaction {
   @_ = (@$a);
   my $tag = shift;
   my $maniere = shift;
-  my $step = 50;
+  my $step = Tk::SlideShow->steps;
   $maniere eq 'smooth'  and 
     do {
       my ($dx,$dy) = @_;
       for(my $i=0;$i<$step;$i++){
 	if (ref($tag) eq 'ARRAY') {
-	  for (@$tag) {$c->move($_,-$dx/$step,-$dy/$step);}
-	} else { $c->move($tag,-$dx/$step,-$dy/$step);}
+	  for (@$tag) {
+	    $c->move($_,-$dx/$step,-$dy/$step);
+	    my $spri = Tk::SlideShow::Dict->Get($_);
+	    for my $l ($spri->links) {$l->show;}
+	  }
+	} else { 
+	  $c->move($tag,-$dx/$step,-$dy/$step);
+	  my $spri = Tk::SlideShow::Dict->Get($tag);
+	  for my $l ($spri->links) {$l->show;}
+	}
 	$c->update;
       }
     };
@@ -463,12 +530,14 @@ sub add {
 sub play {
   my ($class,$timetowait) = @_;
   my $current = $present->current;
+  $present->present_start(time);
   my $nbslides = @{$present->{'slides'}};
   while(1) {
     $jumpslide = 0;
     $current =  $present->current;
     my $diapo = $present->{'slides'}[$current];
     print "Executing slide number $current\n";
+    $present->diapo_start(time);
     $present->start_slide;
     &{$diapo->code};
     if (defined $timetowait) {
@@ -609,6 +678,90 @@ sub latexabstract {
   close OUT;
 }
 
+sub default_footer {
+  my $now = time;
+#  print "default footer displaying\n";
+#  my $td = $now - $present->diapo_start;
+#  my $tp = $now - $present->present_start;
+  my $num = $present->current+1;
+  my $nbs = $present->nbslides;
+  my $name = $present->currentName;
+#  $td = $td>60 ? sprintf("%s'%ss",int($td/60),$td%60) : "${td}s";
+#  $tp = $tp>60 ? sprintf("%s'%ss",int($tp/60),$tp%60) : "${tp}s";
+    
+#  my $t = "$name($num($td))/$nbs($tp))";
+  my $t = "$name($num/$nbs)";
+  $can->delete('footer');
+  $can->createText(10,$H - 10,'-text',$t,-anchor,'sw',
+		   -tags,'footer');
+}
+
+sub template {
+  print q#!/usr/local/bin/perl5
+
+use Tk::SlideShow;
+use strict;
+
+my $p = Tk::SlideShow->init(1024,768) or die;
+
+$p->save;
+
+my ($mw,$c,$h,$w) = ($p->mw, $p->canvas, $p->h, $p->w);
+my $d;
+
+#--------------------------------------------
+$d = $p->add('summary',
+	     sub {
+	       title('First title');
+	       my @ids = items('a0',"item1 \n item2 \n item3",
+			       -font => $p->f2,-fill, 'red');
+	       $p->load;
+	       $p->a_top(@ids);
+	     });
+
+$d->html(" ");
+
+#--------------------------------------------
+
+sub title { $p->Text('title',shift,-font,$p->f3); }
+
+sub items {
+  my ($id,$items,@options) = @_; my @ids;
+  for (split (/\n/,$items)) {
+    s/^\s*//; s/\s*$//;
+    $p->Text($id,$_,@options);
+    push @ids,$id; $id++;
+  }
+  return @ids;
+}
+sub example {
+  my ($id,$t,@options) = @_;
+  $t =~ s/^\s+//; $t =~ s/\s+$//;
+  my $s = $p->newSprite($id);
+  my $f = $c->Font('family'  => "courier", point => 250, -weight => 'bold');
+  $c->createText(0,0,-text,'Example',
+		 -font => $f, -tags => $id, 
+		 -fill,'red',
+		 -anchor => 'sw');
+  my $idw = $c->createText(0,0,-text,$t,@options, -tags => $id,
+			   -fill,'yellow', -font => $f,
+			  -anchor => 'nw');
+  $c->createRectangle($c->bbox($idw), -fill,'black',-tags => $id);
+  $c->raise($idw);
+  $s->pan(1);
+  return $s;
+}
+
+
+if (grep (/-html/,@ARGV)) {
+  $p->html("doc");
+  exit 0;
+}
+
+$p->current(shift || 0);
+$p->play;
+;
+}
 
 # wrappers
 
@@ -623,6 +776,7 @@ sub Text {return Tk::SlideShow::Sprite::text(@_);}
 sub Framed {return Tk::SlideShow::Sprite::framed(@_);}
 sub Image {return Tk::SlideShow::Sprite::image(@_);}
 sub Anim {return Tk::SlideShow::Sprite::anim(@_);}
+sub Oval {return Tk::SlideShow::Oval::New(@_);}
 
 sub TickerTape {return Tk::SlideShow::Sprite::tickertape(@_);}
 sub Compuman {return Tk::SlideShow::Sprite::compuman(@_);}
