@@ -16,17 +16,20 @@ use vars qw(@ISA @classes);
 sub New {
   my ($class,$id) = @_;
   die "An mandatory id is needed !" unless defined $id;
-  my $s = bless {'x'=>Tk::SlideShow->w/2,'y'=>Tk::SlideShow->h/2,'id'=>$id};
+  my $s = bless {'x'=>Tk::SlideShow->w/2,'y'=>Tk::SlideShow->h/2,'id'=>$id,
+		'link' => []};
   $class->Set($id,$s);
   return $s;
 }
 
 sub x { return (shift)->{'x'};}
 sub y { return (shift)->{'y'};}
+sub xy { my $s = shift; return ($s->{'x'},$s->{'y'})};
 
 sub no { 
   my $s = shift;
   my ($x1,$y1,$x2,$y2) = Tk::SlideShow->canvas->bbox($s->id);
+#  print "bbox(".$s->id." = $x1,$y1,$x2,$y2\n";
   return ($x1,$y1);
 }
 sub n { 
@@ -69,6 +72,7 @@ sub pos {
   my $s = shift;
   my $pos = shift;
   $pos = ( $pos + 8 ) % 8;
+#  print "returning pos of ".$s->id." at $pos\n";
   return $s->no if $pos == 0;
   return $s->n  if $pos == 1;
   return $s->ne if $pos == 2;
@@ -90,12 +94,6 @@ sub links { return @{(shift)->{'link'}};}
 
 sub id { return (shift)->{'id'};}
 
-
-sub AddClass{
-  shift;
-  push @classes,@_;
-}
-
 sub evalplace {
   my $s = shift;
   die "La mthode 'evalplace' doit tre redfinie pour la classe ".ref($s)." et ne l'est pas, apparament\n";
@@ -111,15 +109,13 @@ sub save {
   my $dfltfname = "slide-$numero.pl";
   my %files = ();
   print "saving slide $numero\n";
-  foreach my $cl (@classes) {
-    #print "Scanning class $cl\n";
-    while (my $s = $cl->Each) {
-      my $id = $s->id;
-      next if $id eq '__null__';
-      $id =~ s/['\\]/\\$&/g;
-      my $fname = ($id =~ m|/|) ? $`: $dfltfname ;
-      $files{$fname} .= "$cl->Get('$id')->".$s->evalplace.";\n";
-    }
+  for my $id ( Tk::SlideShow::Dict->All) {
+    my $s = Tk::SlideShow::Dict->Get($id);
+#    print "Saving $id of class :".ref($s)."\n";
+    next if $id eq '__null__';
+    $id =~ s/['\\]/\\$&/g;
+    my $fname = ($id =~ m|/|) ? $`: $dfltfname ;
+    $files{$fname} .= "Tk::SlideShow::Dict->Get('$id')->".$s->evalplace.";\n";
   }
   while(my($k,$v) = each %files) {
     print "Generating file  $k\n";
@@ -131,11 +127,7 @@ sub save {
 
 
 sub Clean {
-  foreach my $cl (@classes) {
-    while (my $s = $cl->Each) {
-      $cl->Del($s->id);
-    }
-  }
+  Tk::SlideShow::Dict->Clean;
 }
 
 
@@ -152,12 +144,14 @@ sub pan {
   $c->bind($id,"<$button>", 
 	   sub { 
 	       my $e = (shift)->XEvent;
-#	       my $id = $s->id;
+	       #	       my $id = $s->id;
+#	       print "B1 pressed\n";
 	       $c->raise($id);
 	       ($s->{'sx'},$s->{'sy'}) = ($c->canvasx($e->x),$c->canvasy($e->y));
 	     });
   $c->bind($id,"<B$button-Motion>", 
 	   sub {
+#	     print "B1 motion\n";
 	     my $e = (shift)->XEvent;
 #	     my $id = $s->id;
 	     my ($nx,$ny) = ($c->canvasx($e->x),$c->canvasy($e->y));
@@ -169,6 +163,15 @@ sub pan {
 
 	   });
   return $s;
-}       
+}
+
+sub cursor {
+  my ($s,$cursorname) = @_;
+  my $c = Tk::SlideShow->canvas;
+  my $id = $s->id;
+  $c->bind($id, '<Enter>',sub { $c->configure(-cursor,$cursorname)});
+  $c->bind($id, '<Leave>',sub { $c->configure(-cursor,'top_left_arrow')});
+}
+
 
 1;
