@@ -5,7 +5,7 @@ use strict;
 	
 my $p = init Tk::SlideShow(1024,768);
 $p->save;
-my ($mw,$c,$h,$w) = ($p->mw, $p->can, $p->h, $p->w);
+my ($mw,$c,$h,$w) = ($p->mw, $p->canvas, $p->h, $p->w);
 
 sub bandeau {
   my ($id,$text,$len) = @_;
